@@ -23,6 +23,8 @@
 #include "simfilesystem.h"
 #include "simapplication.h"
 #include "callmanager.h"
+#include "simauth.h"
+#include "aidapplication.h"
 #include <qatutils.h>
 
 #include <qstring.h>
@@ -268,8 +270,12 @@ SimChat::SimChat( SimState *state, SimXmlNode& e )
         } else if ( n->tag == "switch" ) {
             switchTo = n->getAttribute( "name" );
         } else if ( n->tag == "set" ) {
-	    variables += n->getAttribute( "name" );
+            QString name = n->getAttribute( "name" );
+	    variables += name;
 	    values += n->getAttribute( "value" );
+            int delay = n->getAttribute( "delay" ).toInt();
+            if (delay)
+                delays[name] = delay;
         } else if ( n->tag == "newcall" ) {
             newCallVar = n->getAttribute( "name" );
         } else if ( n->tag == "forgetcall" ) {
@@ -331,22 +337,36 @@ bool SimChat::command( const QString& cmd )
     for ( int varNum = 0; varNum < variables.size(); ++varNum ) {
     	QString variable = variables[varNum];
 	QString value = values[varNum];
-        if ( value != "*" ) {
+        int delay = delays.value(variable, 0);
+        QString val;
+
+        if ( value == "*" )
+            val = wild;
+        else {
             int index = value.indexOf( "${*}" );
-            if ( index == -1 ) {
-                state()->rules()->setVariable( variable, value );
-            } else {
+            if ( index != -1 ) {
                 if ( wild.length() > 0 && wild[wild.length() - 1] == 0x1A ) {
                     // Strip the terminating ^Z from SMS PDU's.
                     wild = wild.left( wild.length() - 1 );
                 }
-                state()->rules()->setVariable
-                    ( variable, value.left( index ) + wild +
-                    value.mid( index + 4 ) );
-            }
-        } else {
-            state()->rules()->setVariable( variable, wild );
+
+                val = value.left( index ) + wild + value.mid( index + 4 );
+            } else
+                val = value;
         }
+
+        if (delay) {
+            QVariantTimer *timer = new QVariantTimer(this->state()->rules());
+
+            timer->param = QVariant::fromValue(QPairKV(variable, val));
+
+            timer->setSingleShot( true );
+
+            connect(timer, SIGNAL(timeout()), this->state()->rules(),
+                    SLOT(delaySetVariable()));
+            timer->start( delay );
+        } else
+            state()->rules()->setVariable( variable, val );
     }
 
     // Switch to the new state.
@@ -530,6 +550,12 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
                  _callManager, SLOT(startIncomingCall(QString,QString,QString)) );
         connect ( _callManager, SIGNAL( callStatesChanged( QList<CallInfo> * ) ),
                   machine, SLOT( callManagement( QList<CallInfo> * ) ) );
+        connect ( machine, SIGNAL( stateChangedToAlerting() ), _callManager,
+                SLOT( dialingToAlerting() ) );
+        connect ( machine, SIGNAL( stateChangedToConnected() ), _callManager,
+                SLOT( dialingToConnected() ) );
+        connect ( machine, SIGNAL( stateChangedToHangup( int ) ), _callManager,
+                SLOT( hangupRemote( int ) ) );
     }
 
     connect(this,SIGNAL(readyRead()),
@@ -539,8 +565,6 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
     // Initialize the local state.
     currentState = 0;
     defState = 0;
-    return_error_string = "";
-    return_error_count = 0;
     usedCallIds = 0;
     fileSystem = 0;
     useGsm0710 = false;
@@ -606,10 +630,20 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
             // Load a phonebook definition.
             loadPhoneBook( *n );
 
+        } else if ( n->tag == "simauth" ) {
+
+            _simAuth = new SimAuth( this, *n );
+
+        } else if ( n->tag == "application" ) {
+            AidApplication *app = new AidApplication( this, *n );
+            _applications.append(app);
         }
         n = n->next;
     }
 
+    if ( _applications.length() > 0 )
+        _app_wrapper = new AidAppWrapper( this, _applications, _simAuth );
+
     // Clean up the XML reader objects.
     delete handler;
 
@@ -864,6 +898,10 @@ void SimRules::destruct()
         delete defState;
     defState = NULL;
 
+    if ( _simAuth )
+        delete _simAuth;
+    _simAuth = NULL;
+
     if ( _callManager )
         delete _callManager;
     _callManager = NULL;
@@ -957,16 +995,27 @@ bool SimRules::simCsimOk( const QByteArray& payload )
 
 bool SimRules::simCommand( const QString& cmd )
 {
-    /* Process SIM toolkit begin and end commands by forcing the
-     * app back to the main menu.  */
-    if ( cmd == "AT*TSTB" || cmd == "AT*TSTE" ) {
-        if ( !toolkitApp ) {
+    // 3GPP Terminal Response Command
+    if ( cmd.startsWith("AT+CUSATT=") ) {
+        int start = cmd.indexOf( QChar('=') ) + 1;
+        QByteArray response = QAtUtils::fromHex( cmd.mid(start) );
+        QSimTerminalResponse resp = QSimTerminalResponse::fromPdu( response );
+
+        if ( !toolkitApp || !toolkitApp->response( resp ) )
             respond( "ERROR" );
-            return true;
-        }
+
+        return true;
+    }
+
+    // 3GPP Envelope command
+    if ( cmd.startsWith("AT+CUSATE=") ) {
+        int start = cmd.indexOf( QChar('=') ) + 1;
+        QByteArray envelope = QAtUtils::fromHex( cmd.mid(start) );
+        QSimEnvelope env = QSimEnvelope::fromPdu( envelope );
+
+        if (!toolkitApp || !toolkitApp->envelope( env ) )
+            respond( "ERROR" );
 
-        respond( "OK" );
-        toolkitApp->abort();
         return true;
     }
 
@@ -1095,6 +1144,9 @@ void SimRules::command( const QString& cmd )
     if ( _callManager->command( cmd ) )
         return;
 
+    if (_app_wrapper && _app_wrapper->command( cmd ))
+        return;
+
     // Process SIM toolkit related commands with the current SIM application.
     if ( simCommand( cmd ) )
         return;
@@ -1504,13 +1556,6 @@ SimPhoneBook *SimRules::currentPB() const
         return 0;
 }
 
-void SimRules::setReturnError( const QString &error, uint repeat )
-{
-    return_error_string = error;
-    return_error_count = repeat;
-}
-
-
 int SimRules::newCall()
 {
     int id;
@@ -1585,15 +1630,8 @@ QString expandEscapes( const QString& data, bool eol )
 void SimRules::respond( const QString& resp, int delay, bool eol )
 {
     QString r = expand( resp );
-    if (return_error_string != "") {
-        r = return_error_string;
-        if (return_error_count > 0)
-            return_error_count--;
-        if (return_error_count == 0)
-            return_error_string = "";
-    }
-
     QByteArray escaped = expandEscapes( r, eol ).toUtf8();
+
     if ( !delay ) {
         writeChatData(escaped.data(), escaped.length());
         flush();
@@ -1609,7 +1647,12 @@ void SimRules::respond( const QString& resp, int delay, bool eol )
 
 void SimRules::proactiveCommandNotify( const QByteArray& cmd )
 {
-    unsolicited( "*TCMD: " + QString::number( cmd.size() ) );
+    unsolicited( "+CUSATP: " + QAtUtils::toHex( cmd ) );
+}
+
+void SimRules::modemHandledCommandNotify( const QByteArray& cmd )
+{
+    unsolicited( "*HCMD: " + QAtUtils::toHex( cmd ) );
 }
 
 void SimRules::callControlEventNotify( const QSimControlEvent& evt )
@@ -1629,6 +1672,15 @@ void SimRules::delayTimeout()
     timer->deleteLater();
 }
 
+void SimRules::delaySetVariable()
+{
+    QVariantTimer *timer = (QVariantTimer *)sender();
+    QPairKV kv = timer->param.value<QPairKV>();
+
+    setVariable( kv.first, kv.second );
+
+    delete timer;
+}
 
 void SimRules::dialCheck( const QString& number, bool& ok )
 {
